Attribute VB_Name = "HSLRoutines"
Attribute VB_Description = "Wichtige HSL Umrechnungsfunktionen"
Option Explicit

'Conversion to VB and !!!ORIGINAL!!! code by
'Dan Redding (bwsoft@revealed.net)
'http://home.revealed.net/bwsoft

Public Const HSLMAX As Integer = 240
'Kann beliebig gendert werden. Wird Global
'untersttzt.

Public Const RGBMAX As Integer = 255
Public Const UNDEFINED As Integer = -1 'Nachtrglich gendert!
'Wenn -1, dann wird spter der alte Hue Wert ausgegeben!

Public Function iMax(ByVal a As Long, ByVal B As Long) As Long
    iMax = IIf(a > B, a, B)
End Function

Public Function iMin(ByVal a As Long, ByVal B As Long) As Long
    iMin = IIf(a < B, a, B)
End Function

Public Function cRGBtoHSL(RGBCol As Long) As HSLColT
    'Returns an HSLColor datatype containing H
    '     ue, Luminescence
    'and Saturation; given an RGB Color valu
    '     e
    Dim R As Integer, G As Integer, B As Integer
    Dim cMax As Integer, cMin As Integer
    Dim RDelta As Double, GDelta As Double, _
        BDelta As Double
    Dim H As Double, S As Double, L As Double
    Dim cMinus As Long, cPlus As Long

    R = getRedVal(RGBCol)
    G = getGreenVal(RGBCol)
    B = getBlueVal(RGBCol)

    cMax = iMax(iMax(R, G), B) 'Highest and lowest
    cMin = iMin(iMin(R, G), B) 'color values

    cMinus = cMax - cMin 'Used to simplify the
    cPlus = cMax + cMin 'calculations somewhat.

    'Calculate luminescence (lightness)
    L = ((cPlus * HSLMAX) + RGBMAX) / (2 * RGBMAX)

    If cMax = cMin Then 'achromatic (r=g=b, greyscale)
        S = 0 'Saturation 0 for greyscale
        H = UNDEFINED 'Hue undefined for greyscale
    Else
        'Calculate color saturation

        If L <= (HSLMAX / 2) Then
            S = ((cMinus * HSLMAX) + 0.5) / cPlus
        Else
            S = ((cMinus * HSLMAX) + 0.5) / (2 * RGBMAX - cPlus)
        End If

        'Calculate hue
        RDelta = (((cMax - R) * (HSLMAX / 6)) + 0.5) / cMinus
        GDelta = (((cMax - G) * (HSLMAX / 6)) + 0.5) / cMinus
        BDelta = (((cMax - B) * (HSLMAX / 6)) + 0.5) / cMinus

        Select Case cMax
            Case CLng(R)
                H = BDelta - GDelta
            Case CLng(G)
                H = (HSLMAX / 3) + RDelta - BDelta
            Case CLng(B)
                H = ((2 * HSLMAX) / 3) + GDelta - RDelta
        End Select

        If H < 0 Then H = H + HSLMAX
    End If

    cRGBtoHSL.Hue = CInt(H)
    cRGBtoHSL.Lum = CInt(L)
    cRGBtoHSL.Sat = CInt(S)
End Function

Public Function cHSLtoRGB(HueLumSat As HSLColT) As Long '***
    Dim R As Long, G As Long, B As Long
    Dim H As Long, L As Long, S As Long
    Dim Magic1 As Integer, Magic2 As Integer
    H = HueLumSat.Hue
    L = HueLumSat.Lum
    S = HueLumSat.Sat

    If S = 0 Then 'Greyscale
        R = (L * RGBMAX) / HSLMAX 'luminescence,
        'converted to the proper range
        G = R 'All RGB values same in greyscale
        B = R

        If H <> UNDEFINED Then
            'This is technically an error.
            'The cRGBtoHSL routine will always return
            '
            'Hue = UNDEFINED (in this case 160)
            'when Sat = 0.
            'if you are writing a color mixer and
            'letting the user input color values,
            'you may want to set Hue = UNDEFINED
            'in this case.
        End If
    Else
        'Get the "Magic Numbers"

        If L <= HSLMAX / 2 Then
            Magic2 = (L * (HSLMAX + S) + _
                (HSLMAX / 2)) / HSLMAX
        Else
            Magic2 = L + S - ((L * S) + _
                (HSLMAX / 2)) / HSLMAX
        End If
        Magic1 = 2 * L - Magic2
        'get R, G, B; change units from HSLMAX range
        'to RGBMAX range
        R = (HuetoRGB(Magic1, Magic2, H + (HSLMAX / 3)) _
            * RGBMAX + (HSLMAX / 2)) / HSLMAX
        G = (HuetoRGB(Magic1, Magic2, H) _
            * RGBMAX + (HSLMAX / 2)) / HSLMAX
        B = (HuetoRGB(Magic1, Magic2, H - (HSLMAX / 3)) _
            * RGBMAX + (HSLMAX / 2)) / HSLMAX
    End If
    cHSLtoRGB = RGB(CInt(R), CInt(G), CInt(B))
End Function

Private Function HuetoRGB(mag1 As Integer, mag2 As Integer, _
    Hue As Long) As Long '***
    'Utility function for cHSLtoRGB
    'Range check

    If Hue < 0 Then
        Hue = Hue + HSLMAX
    ElseIf Hue > HSLMAX Then
        Hue = Hue - HSLMAX
    End If
    'Return r, g, or b value from parameters
    '

    Select Case Hue 'Values get progressively larger.
        'Only the first true condition will exec
        '     ute
        Case Is < (HSLMAX / 6)
            HuetoRGB = (mag1 + (((mag2 - mag1) * Hue + _
                (HSLMAX / 12)) / (HSLMAX / 6)))
        Case Is < (HSLMAX / 2)
            HuetoRGB = mag2
        Case Is < (HSLMAX * 2 / 3)
            HuetoRGB = (mag1 + (((mag2 - mag1) * _
                ((HSLMAX * 2 / 3) - Hue) + _
                (HSLMAX / 12)) / (HSLMAX / 6)))
        Case Else
            HuetoRGB = mag1
    End Select
End Function

